@@ -52,8 +52,8 @@ def ais_to_nmea_0183(payload: str, ais_talker_id: str, radio_channel: str, fill_
     """
     messages = []
     max_len = 61
-    seq_id = ''
     frag_cnt = math.ceil(len(payload) / max_len)
+    seq_id = '0' if frag_cnt > 1 else ''
 
     if len(ais_talker_id) != 5:
         raise ValueError("AIS talker is must have exactly 6 characters. E.g. AIVDO")
@@ -63,9 +63,10 @@ def ais_to_nmea_0183(payload: str, ais_talker_id: str, radio_channel: str, fill_
 
     for frag_num, chunk in enumerate(chunks(payload, max_len), start=1):
         tpl = "!{},{},{},{},{},{},{}*{:02X}"
-        dummy_message = tpl.format(ais_talker_id, frag_cnt, frag_num, seq_id, radio_channel, chunk, fill_bits, 0)
+        fill_bits_frag = fill_bits if frag_num == frag_cnt else 0  # Make sure we set fill bits only for last fragment
+        dummy_message = tpl.format(ais_talker_id, frag_cnt, frag_num, seq_id, radio_channel, chunk, fill_bits_frag, 0)
         checksum = compute_checksum(dummy_message)
-        msg = tpl.format(ais_talker_id, frag_cnt, frag_num, seq_id, radio_channel, chunk, fill_bits, checksum)
+        msg = tpl.format(ais_talker_id, frag_cnt, frag_num, seq_id, radio_channel, chunk, fill_bits_frag, checksum)
         messages.append(msg)
 
     return messages
