@@ -11,7 +11,7 @@ from pyais.constants import TalkerID, NavigationStatus, ManeuverIndicator, EpfdT
     TransmitMode, StationIntervals
 from pyais.exceptions import InvalidNMEAMessageException, UnknownMessageException, UnknownPartNoException, \
     InvalidDataTypeException
-from pyais.util import decode_into_bit_array, compute_checksum, int_to_bin, str_to_bin, \
+from pyais.util import decode_into_bit_array, compute_checksum, get_itdma_comm_state, get_sotdma_comm_state, int_to_bin, str_to_bin, \
     encode_ascii_6, from_bytes, from_bytes_signed, decode_bin_as_ascii6, get_int, chk_to_int, coerce_val, \
     bits2bytes, bytes2bits, b64encode_str
 
@@ -160,6 +160,7 @@ class NMEAMessage(object):
         'payload',
         'fill_bits',
         'checksum',
+        'is_valid',
         'bit_array'
     )
 
@@ -217,6 +218,9 @@ class NMEAMessage(object):
         self.bit_array: bitarray = decode_into_bit_array(self.payload, self.fill_bits)
         self.ais_id: int = get_int(self.bit_array, 0, 6)
 
+        # Set the checksum valid field
+        self.is_valid = self.checksum == compute_checksum(self.raw)
+
     def __str__(self) -> str:
         return str(self.raw)
 
@@ -247,6 +251,7 @@ class NMEAMessage(object):
             'fill_bits': self.fill_bits,  # int
             'checksum': self.checksum,  # int
             'bit_array': self.bit_array.to01(),  # str
+            'is_valid': self.is_valid,  # bool
         }
 
     def decode_and_merge(self, enum_as_int: bool = False) -> Dict[str, Any]:
@@ -283,6 +288,7 @@ class NMEAMessage(object):
         raw = b''
         data = b''
         bit_array = bitarray()
+        is_valid = True
 
         for i, msg in enumerate(sorted(messages, key=lambda m: m.frag_num)):
             if i > 0:
@@ -290,16 +296,14 @@ class NMEAMessage(object):
             raw += msg.raw
             data += msg.payload
             bit_array += msg.bit_array
+            is_valid &= msg.is_valid
 
         messages[0].raw = raw
         messages[0].payload = data
         messages[0].bit_array = bit_array
+        messages[0].is_valid = is_valid
         return messages[0]
 
-    @property
-    def is_valid(self) -> bool:
-        return self.checksum == compute_checksum(self.raw)
-
     @property
     def is_single(self) -> bool:
         return not self.seq_id and self.frag_num == self.frag_cnt == 1
@@ -481,14 +485,14 @@ class Payload(abc.ABC):
         """
         if enum_as_int:
             d: typing.Dict[str, typing.Optional[NMEA_VALUE]] = {}
-            for slt in self.__slots__:
+            for slt in self.__slots__:  # type: ignore
                 val = getattr(self, slt)
                 if val is not None and slt in ENUM_FIELDS:
                     val = int(getattr(self, slt))
                 d[slt] = val
             return d
         else:
-            return {slt: getattr(self, slt) for slt in self.__slots__}
+            return {slt: getattr(self, slt) for slt in self.__slots__}  # type: ignore
 
     def to_json(self) -> str:
         return JSONEncoder(indent=4).encode(self.asdict())
@@ -540,7 +544,7 @@ def to_turn(turn: typing.Union[int, float]) -> typing.Optional[float]:
     elif abs(turn) == 127 or abs(turn) == 128:
         return None
     else:
-        return math.copysign((round(turn / 4.733)) ** 2, turn)
+        return math.copysign(int((turn / 4.733) ** 2), turn)
 
 
 def from_turn(turn: typing.Optional[typing.Union[int, float]]) -> int:
@@ -549,8 +553,62 @@ def from_turn(turn: typing.Optional[typing.Union[int, float]]) -> int:
     return int(math.copysign(round(4.733 * math.sqrt(abs(turn))), turn))
 
 
+class CommunicationStateMixin:
+    """
+    Mixin class to access Communication State values by applicable messages.
+
+    You may refer to 3.3.7.2.1 of:
+    https://www.itu.int/dms_pubrec/itu-r/rec/m/R-REC-M.1371-1-200108-S!!PDF-E.pdf
+    """
+
+    radio: int  # Type hint to make mypy happy
+
+    MAX_COMM_STATE_VALUE = 0x7ffff
+
+    def get_communication_state(self) -> Dict[str, typing.Optional[int]]:
+        result: Dict[str, typing.Optional[int]] = {
+            'received_stations': None,
+            'slot_number': None,
+            'utc_hour': None,
+            'utc_minute': None,
+            'slot_offset': None,
+            'slot_timeout': None,
+            'sync_state': None,
+            'keep_flag': None,
+            'slot_increment': None,
+            'num_slots': None,
+        }
+
+        if self.is_sotdma:
+            result.update(get_sotdma_comm_state(self.communication_state_raw))
+        else:
+            result.update(get_itdma_comm_state(self.communication_state_raw))
+
+        return result
+
+    @property
+    def is_sotdma(self) -> bool:
+        """The radio status field has it's 20th bit (MSB) set to 0 or has less than 20 bits"""
+        return self.radio <= self.MAX_COMM_STATE_VALUE
+
+    @property
+    def is_itdma(self) -> bool:
+        """The radio status field has it's 20th bit (MSB) set to 1"""
+        return self.radio > self.MAX_COMM_STATE_VALUE
+
+    @property
+    def communication_state_raw(self) -> int:
+        """Get the raw radio status except 20th bit - if present"""
+        try:
+            return self.radio & self.MAX_COMM_STATE_VALUE
+        except AttributeError as err:
+            raise ValueError(
+                'Communication State is only available for messages with radio field'
+            ) from err
+
+
 @attr.s(slots=True)
-class MessageType1(Payload):
+class MessageType1(Payload, CommunicationStateMixin):
     """
     AIS Vessel position report using SOTDMA (Self-Organizing Time Division Multiple Access)
     Src: https://gpsd.gitlab.io/gpsd/AIVDM.html#_types_1_2_and_3_position_report_class_a
@@ -591,7 +649,7 @@ class MessageType3(MessageType1):
 
 
 @attr.s(slots=True)
-class MessageType4(Payload):
+class MessageType4(Payload, CommunicationStateMixin):
     """
     AIS Vessel position report using SOTDMA (Self-Organizing Time Division Multiple Access)
     Src: https://gpsd.gitlab.io/gpsd/AIVDM.html#_type_4_base_station_report
@@ -698,7 +756,7 @@ class MessageType8(Payload):
 
 
 @attr.s(slots=True)
-class MessageType9(Payload):
+class MessageType9(Payload, CommunicationStateMixin):
     """
     Standard SAR Aircraft Position Report
     Src: https://gpsd.gitlab.io/gpsd/AIVDM.html#_type_9_standard_sar_aircraft_position_report
@@ -843,7 +901,7 @@ class MessageType17(Payload):
 
 
 @attr.s(slots=True)
-class MessageType18(Payload):
+class MessageType18(Payload, CommunicationStateMixin):
     """
     Standard Class B CS Position Report
     Src: https://gpsd.gitlab.io/gpsd/AIVDM.html#_type_18_standard_class_b_cs_position_report
@@ -1247,7 +1305,7 @@ class MessageType25(Payload):
 
 
 @attr.s(slots=True)
-class MessageType26AddressedStructured(Payload):
+class MessageType26AddressedStructured(Payload, CommunicationStateMixin):
     msg_type = bit_field(6, int, default=26, signed=False)
     repeat = bit_field(2, int, default=0, signed=False)
     mmsi = bit_field(30, int, from_converter=from_mmsi)
@@ -1262,7 +1320,7 @@ class MessageType26AddressedStructured(Payload):
 
 
 @attr.s(slots=True)
-class MessageType26BroadcastStructured(Payload):
+class MessageType26BroadcastStructured(Payload, CommunicationStateMixin):
     msg_type = bit_field(6, int, default=26, signed=False)
     repeat = bit_field(2, int, default=0, signed=False)
     mmsi = bit_field(30, int, from_converter=from_mmsi)
@@ -1276,7 +1334,7 @@ class MessageType26BroadcastStructured(Payload):
 
 
 @attr.s(slots=True)
-class MessageType26AddressedUnstructured(Payload):
+class MessageType26AddressedUnstructured(Payload, CommunicationStateMixin):
     msg_type = bit_field(6, int, default=26, signed=False)
     repeat = bit_field(2, int, default=0, signed=False)
     mmsi = bit_field(30, int, from_converter=from_mmsi)
@@ -1291,7 +1349,7 @@ class MessageType26AddressedUnstructured(Payload):
 
 
 @attr.s(slots=True)
-class MessageType26BroadcastUnstructured(Payload):
+class MessageType26BroadcastUnstructured(Payload, CommunicationStateMixin):
     msg_type = bit_field(6, int, default=26, signed=False)
     repeat = bit_field(2, int, default=0, signed=False)
     mmsi = bit_field(30, int, from_converter=from_mmsi)
