@@ -9,10 +9,10 @@ from pprint import pprint
 from pyais import NMEAMessage, encode_dict
 from pyais.ais_types import AISType
 from pyais.constants import (EpfdType, ManeuverIndicator, NavAid,
-                             NavigationStatus, ShipType, StationType,
+                             NavigationStatus, ShipType, StationType, SyncState,
                              TransmitMode)
 from pyais.decode import decode
-from pyais.exceptions import UnknownMessageException
+from pyais.exceptions import InvalidNMEAChecksum, UnknownMessageException
 from pyais.messages import (MSG_CLASS, MessageType5, MessageType6,
                             MessageType18, MessageType22Addressed,
                             MessageType22Broadcast, MessageType24PartA,
@@ -270,6 +270,19 @@ class TestAIS(unittest.TestCase):
 
         ensure_type_for_msg_dict(msg)
 
+    def test_msg_type_8_multipart(self):
+        msgs = ['!AIVDO,2,1,,A,8=?eN>0000:C=4B1KTTsgLoUelGetEo0FoWr8jo=?045TNv5Tge6sAUl4MKWo,0*5F', '!AIVDO,2,2,,A,vhOL9NIPln:BsP0=BLOiiCbE7;SKsSJfALeATapHfdm6Tl,2*79']
+
+        msg = decode(*msgs).asdict()
+
+        assert msg["repeat"] == 0
+        assert msg["mmsi"] == 888888888
+        assert msg['dac'] == 0
+        assert msg["fid"] == 0
+        assert msg["data"] == b"\x02\x934D\x81nI;\xbd\xcd\xe5\xb7E\xed\xf1]\xc0[y\xfa#-\xcd<\x01\x05\x91\xef\x85\x92\xfbF\xed\x19t\x11\xd6\xe7\xdf\xec\x1fp\x97\x99\x83M\x8aK\xb8\x005'\x1f\xc7\x14\xeaTr\xe3o\xb8\xda\xb9\x17-FJxb\xeb5\x1aM"
+
+        ensure_type_for_msg_dict(msg)
+
     def test_msg_type_9(self):
         msg = decode(b"!AIVDM,1,1,,B,91b55wi;hbOS@OdQAC062Ch2089h,0*30").asdict()
         assert msg['msg_type'] == 9
@@ -911,6 +924,7 @@ class TestAIS(unittest.TestCase):
                 'ais_id': 21,
                 'assigned': None,
                 'channel': 'B',
+                'is_valid': True,
                 'checksum': 82,
                 'epfd': 7,
                 'fill_bits': 2,
@@ -948,6 +962,7 @@ class TestAIS(unittest.TestCase):
                 'assigned': None,
                 'channel': 'B',
                 'checksum': 82,
+                'is_valid': True,
                 'epfd': EpfdType.Surveyed,
                 'fill_bits': 2,
                 'frag_cnt': 1,
@@ -1227,32 +1242,209 @@ class TestAIS(unittest.TestCase):
     def test_rot_encode_yields_expected_values(self):
         encoded = encode_dict({'msg_type': 1, 'mmsi': 123, 'turn': 25.0})[0]
         assert encoded == "!AIVDO,1,1,,A,10000Nh600000000000000000000,0*05"
-        assert decode(encoded).turn == 25.0
 
         encoded = encode_dict({'msg_type': 1, 'mmsi': 123, 'turn': -16.0})[0]
         assert encoded == "!AIVDO,1,1,,A,10000Nhs@0000000000000000000,0*30"
-        assert decode(encoded).turn == -16.0
 
         encoded = encode_dict({'msg_type': 1, 'mmsi': 123, 'turn': 4.0})[0]
         assert encoded == "!AIVDO,1,1,,A,10000Nh2@0000000000000000000,0*71"
-        assert decode(encoded).turn == 4.0
 
         encoded = encode_dict({'msg_type': 1, 'mmsi': 123, 'turn': -4.0})[0]
         assert encoded == "!AIVDO,1,1,,A,10000Nhuh0000000000000000000,0*1E"
-        assert decode(encoded).turn == -4.0
 
         encoded = encode_dict({'msg_type': 1, 'mmsi': 123, 'turn': -121.0})[0]
         assert encoded == "!AIVDO,1,1,,A,10000Nhk00000000000000000000,0*58"
-        assert decode(encoded).turn == -121.0
 
         encoded = encode_dict({'msg_type': 1, 'mmsi': 123, 'turn': 64.0})[0]
         assert encoded == "!AIVDO,1,1,,A,10000Nh9P0000000000000000000,0*6A"
-        assert decode(encoded).turn == 64.0
 
     def test_rot_decode_yields_expected_values(self):
         assert decode(b"!AIVDM,1,1,,A,14QIG<5620KF@Gl:L9DI4o8N0P00,0*28").turn == 25.0
-        assert decode(b"!AIVDM,1,1,,B,13u><=gsQj0mQW:Q1<wRL28P0@:4,0*32").turn == -16.0
-        assert decode(b"!AIVDM,1,1,,A,14SSRt021O0?bK@MO7H6QUA600Rg,0*12").turn == 4.0
-        assert decode(b"!AIVDM,1,1,,2,13aB:Hhuh0PHjEFNKJg@11sH08J=,0*1E").turn == -4.0
-        assert decode(b"!AIVDM,1,1,,A,16:VFv0k0I`KQPpFATG4SgvT40:v,0*7B").turn == -121.0
-        assert decode(b"!AIVDM,1,1,,B,16:D3F0:15`5ogh<O?bk>1Dd2L1<,0*0B").turn == 64.0
+        assert decode(b"!AIVDM,1,1,,B,13u><=gsQj0mQW:Q1<wRL28P0@:4,0*32").turn == -14.0
+        assert decode(b"!AIVDM,1,1,,A,14SSRt021O0?bK@MO7H6QUA600Rg,0*12").turn == 2.0
+        assert decode(b"!AIVDM,1,1,,2,13aB:Hhuh0PHjEFNKJg@11sH08J=,0*1E").turn == -3.0
+        assert decode(b"!AIVDM,1,1,,A,16:VFv0k0I`KQPpFATG4SgvT40:v,0*7B").turn == -120.0
+        assert decode(b"!AIVDM,1,1,,B,16:D3F0:15`5ogh<O?bk>1Dd2L1<,0*0B").turn == 71.0
+
+    def test_get_sotdma_comm_state_utc_direct(self):
+        msg = "!AIVDM,1,1,,A,13HOI:0P0000VOHLCnHQKwvL05Ip,0*23"
+        decoded = decode(msg)
+        actual = decoded.get_communication_state()
+
+        assert decoded.is_sotdma
+        assert not decoded.is_itdma
+
+        self.assertEqual(actual, {
+            'received_stations': None,
+            'slot_number': None,
+            'utc_hour': 11,
+            'utc_minute': 30,
+            'slot_offset': None,
+            'slot_timeout': 1,
+            'sync_state': SyncState.UTC_DIRECT,
+            'keep_flag': None,
+            'slot_increment': None,
+            'num_slots': None,
+        })
+
+    def test_get_sotdma_comm_state_utc_direct_slot_number(self):
+        msg = "!AIVDM,1,1,,B,403OtVAv>lba;o?Ia`E`4G?02H6k,0*44"
+        decoded = decode(msg)
+        actual = decoded.get_communication_state()
+
+        assert decoded.is_sotdma
+        assert not decoded.is_itdma
+
+        self.assertEqual(actual, {
+            'received_stations': None,
+            'slot_number': 435,
+            'utc_hour': None,
+            'utc_minute': None,
+            'slot_offset': None,
+            'slot_timeout': 6,
+            'sync_state': SyncState.UTC_DIRECT,
+            'keep_flag': None,
+            'slot_increment': None,
+            'num_slots': None,
+        })
+
+    def test_get_sotdma_comm_state_utc_direct_slot_timeout(self):
+        msg = "!AIVDM,1,1,,B,91b55wi;hbOS@OdQAC062Ch2089h,0*30"
+        decoded = decode(msg)
+        actual = decoded.get_communication_state()
+
+        assert decoded.is_sotdma
+        assert not decoded.is_itdma
+
+        self.assertEqual(actual, {
+            'received_stations': None,
+            'slot_number': 624,
+            'utc_hour': None,
+            'utc_minute': None,
+            'slot_offset': None,
+            'slot_timeout': 2,
+            'sync_state': SyncState.UTC_DIRECT,
+            'keep_flag': None,
+            'slot_increment': None,
+            'num_slots': None,
+        })
+
+    def test_get_comm_state_type_18_itdma_base_indirect(self):
+        msg = '!AIVDM,1,1,,A,B5NJ;PP005l4ot5Isbl03wsUkP06,0*76'
+        decoded = decode(msg)
+
+        assert decoded.communication_state_raw == 393222
+        assert decoded.is_itdma
+        assert not decoded.is_sotdma
+
+        comm_state = decoded.get_communication_state()
+
+        assert isinstance(comm_state, dict)
+        assert comm_state['received_stations'] is None
+        assert comm_state['slot_number'] is None
+        assert comm_state['utc_hour'] is None
+        assert comm_state['utc_minute'] is None
+        assert comm_state['slot_offset'] is None
+        assert comm_state['slot_timeout'] is None
+        assert comm_state['sync_state'] == SyncState.BASE_INDIRECT
+        assert comm_state['keep_flag'] == 0
+        assert comm_state['slot_increment'] == 0
+        assert comm_state['num_slots'] == 3
+
+    def test_get_comm_state_type_18_sotdma_utc_direct(self):
+        msg = '!AIVDM,1,1,,A,B69A5U@3wk?8mP=18D3Q3wSRPD00,0*5C'
+        decoded = decode(msg)
+
+        assert decoded.communication_state_raw == 81920
+        assert not decoded.is_itdma
+        assert decoded.is_sotdma
+
+        comm_state = decoded.get_communication_state()
+
+        assert isinstance(comm_state, dict)
+        assert comm_state['received_stations'] == 0
+        assert comm_state['slot_number'] is None
+        assert comm_state['utc_hour'] is None
+        assert comm_state['utc_minute'] is None
+        assert comm_state['slot_offset'] is None
+        assert comm_state['slot_timeout'] == 5
+        assert comm_state['sync_state'] == SyncState.UTC_DIRECT
+        assert comm_state['keep_flag'] is None
+        assert comm_state['slot_increment'] is None
+        assert comm_state['num_slots'] is None
+
+    def test_get_comm_state_type_18_sotdma_base_inidrect(self):
+        msg = '!AIVDM,1,1,,A,B69Gk3h071tpI02lT2ek?wg61P06,0*1F'
+        decoded = decode(msg)
+
+        assert decoded.communication_state_raw == 393222
+        assert not decoded.is_itdma
+        assert decoded.is_sotdma
+
+        comm_state = decoded.get_communication_state()
+
+        assert isinstance(comm_state, dict)
+        assert comm_state['received_stations'] is None
+        assert comm_state['slot_number'] is None
+        assert comm_state['utc_hour'] is None
+        assert comm_state['utc_minute'] is None
+        assert comm_state['slot_offset'] == 6
+        assert comm_state['slot_timeout'] == 0
+        assert comm_state['sync_state'] == SyncState.BASE_INDIRECT
+        assert comm_state['keep_flag'] is None
+        assert comm_state['slot_increment'] is None
+        assert comm_state['num_slots'] is None
+
+    def test_static_data_report(self):
+        msg_a = b'!ANVDM,1,1,,B,H5NuKGTUCBD8SaUG4:omol0hC33t,0*54'
+        decoded_a = decode(msg_a)
+
+        self.assertEqual(decoded_a.msg_type, 24)
+        self.assertEqual(decoded_a.repeat, 0)
+        self.assertEqual(decoded_a.mmsi, 368008030)
+        self.assertEqual(decoded_a.partno, 1)
+        self.assertEqual(decoded_a.ship_type, ShipType.PleasureCraft)
+        self.assertEqual(decoded_a.vendorid, 'SRT')
+        self.assertEqual(decoded_a.callsign, 'WDJ7574')
+        self.assertEqual(decoded_a.to_bow, 6)
+        self.assertEqual(decoded_a.to_stern, 19)
+        self.assertEqual(decoded_a.to_port, 3)
+        self.assertEqual(decoded_a.to_starboard, 3)
+
+        msg_b = b'!ANVDM,1,1,,A,H5NuKGTUCBD8SaUG4:omol0hC33t,0*57'
+        decoded_b = decode(msg_b)
+
+        self.assertEqual(decoded_a, decoded_b)
+
+    def test_special_position_report(self):
+        msg = b'!ANVDM,1,1,,A,35O5WS1000r9FSHF@jBoLCACp000,0*42'
+        decoded = decode(msg)
+
+        self.assertEqual(decoded.msg_type, 3)
+        self.assertEqual(decoded.repeat, 0)
+        self.assertEqual(decoded.mmsi, 368142220)
+        self.assertEqual(decoded.status, NavigationStatus.AtAnchor)
+        self.assertEqual(decoded.turn, 0)
+        self.assertEqual(decoded.speed, 0)
+        self.assertEqual(decoded.accuracy, 1)
+        self.assertEqual(decoded.lon, -81.84302)
+        self.assertEqual(decoded.lat, 38.906152)
+        self.assertEqual(decoded.course, 190.5)
+        self.assertEqual(decoded.heading, 104)
+        self.assertEqual(decoded.second, 41)
+        self.assertEqual(decoded.raim, 0)
+
+    def test_decode_does_not_raise_an_error_if_error_if_checksum_invalid_is_false(self):
+        raw = b"!AIVDM,1,1,,B,15NG6V0P01G?cFhE`R2IU?wn28R>,0*FF"
+        msg = decode(raw, error_if_checksum_invalid=False)
+        self.assertIsNotNone(msg,)
+
+    def test_decode_does_not_raise_an_error_by_default(self):
+        raw = b"!AIVDM,1,1,,B,15NG6V0P01G?cFhE`R2IU?wn28R>,0*FF"
+        msg = decode(raw)
+        self.assertIsNotNone(msg)
+
+    def test_decode_does_raise_an_error_if_error_if_checksum_invalid_is_true(self):
+        raw = b"!AIVDM,1,1,,B,15NG6V0P01G?cFhE`R2IU?wn28R>,0*FF"
+        with self.assertRaises(InvalidNMEAChecksum):
+            _ = decode(raw, error_if_checksum_invalid=True)
